@@ -1,4 +1,6 @@
-extern crate libc;
+extern crate libloading;
+
+pub use libloading::{Library, Symbol};
 
 #[cfg(feature = "dlopen")]
 #[macro_export]
@@ -40,7 +42,7 @@ macro_rules! link_external_library(
         $(varargs: $(fn $vname: ident($($vargs: ty),+ ...) -> $vret: ty),+)|*
     ) => (
         #[link(name = $link)]
-        extern {
+        extern "C" {
             $($(
                 pub static $sname: $stype;
             )+)*
@@ -67,14 +69,15 @@ macro_rules! dlopen_external_library(
         $(varargs: $(fn $vname: ident($($vargs: ty),+ ...) -> $vret: ty),+)|*
     ) => (
         pub struct $structname {
+            __lib: $crate::Library,
             $($(
-                pub $sname: &'static $stype,
+                pub $sname: $crate::Symbol<'static, &'static $stype>,
             )+)*
             $($(
-                pub $fname: unsafe extern fn($($farg),*) -> $fret,
+                pub $fname: $crate::Symbol<'static, unsafe extern "C" fn($($farg),*) -> $fret>,
             )+)*
             $($(
-                pub $vname: unsafe extern "C" fn($($vargs),+ , ...) -> $vret,
+                pub $vname: $crate::Symbol<'static, unsafe extern "C" fn($($vargs),+ , ...) -> $vret>,
             )+)*
         }
     );
@@ -85,44 +88,39 @@ macro_rules! dlopen_external_library(
     ) => (
     impl $structname {
         pub fn open(name: &str) -> Result<$structname, $crate::DlError> {
-            let cname = match ::std::ffi::CString::new(name) {
-                Ok(cs) => cs,
+            // we use it to ensure the 'static lifetime
+            use std::mem::transmute;
+            let lib = match $crate::Library::new(name) {
+                Ok(l) => l,
                 Err(_) => return Err($crate::DlError::NotFound)
             };
             unsafe {
-                let dl = $crate::ffi::dlopen(cname.as_bytes_with_nul().as_ptr() as *const _, 1);
-                if dl.is_null() {
-                    return Err($crate::DlError::NotFound);
-                }
-                $crate::ffi::dlerror();
                 let s = $structname {
                     $($($sname: {
                         let s_name = concat!(stringify!($sname), "\0");
-                        let symbol = $crate::ffi::dlsym(dl, s_name.as_ptr() as *const _);
-                        if !$crate::ffi::dlerror().is_null() {
-                            return Err($crate::DlError::MissingSymbol(s_name))
-                        }
-                        ::std::mem::transmute(symbol)
+                        transmute(match lib.get::<&'static $stype>(s_name.as_bytes()) {
+                            Ok(s) => s,
+                            Err(_) => return Err($crate::DlError::MissingSymbol(s_name))
+                        })
                     },
                     )+)*
                     $($($fname: {
                         let s_name = concat!(stringify!($fname), "\0");
-                        let symbol = $crate::ffi::dlsym(dl, s_name.as_ptr() as *const _);
-                        if !$crate::ffi::dlerror().is_null() {
-                            return Err($crate::DlError::MissingSymbol(s_name))
-                        }
-                        ::std::mem::transmute(symbol)
+                        transmute(match lib.get::<unsafe extern "C" fn($($farg),*) -> $fret>(s_name.as_bytes()) {
+                            Ok(s) => s,
+                            Err(_) => return Err($crate::DlError::MissingSymbol(s_name))
+                        })
                     },
                     )+)*
                     $($($vname: {
                         let s_name = concat!(stringify!($vname), "\0");
-                        let symbol = $crate::ffi::dlsym(dl, s_name.as_ptr() as *const _);
-                        if !$crate::ffi::dlerror().is_null() {
-                            return Err($crate::DlError::MissingSymbol(s_name))
-                        }
-                        ::std::mem::transmute(symbol)
+                        transmute(match lib.get::<unsafe extern "C" fn($($vargs),+ , ...) -> $vret>(s_name.as_bytes()) {
+                            Ok(s) => s,
+                            Err(_) => return Err($crate::DlError::MissingSymbol(s_name))
+                        })
                     },
                     )+)*
+                    __lib: lib
                 };
                 Ok(s)
             }
@@ -179,13 +177,3 @@ macro_rules! external_library(
         );
     );
 );
-
-pub mod ffi {
-    use libc::{c_char, c_int, c_void};
-    extern {
-        pub fn dlopen(filename: *const c_char, flag: c_int) -> *mut c_void;
-        pub fn dlerror() -> *mut c_char;
-        pub fn dlsym(handle: *mut c_void, symbol: *const c_char) -> *mut c_void;
-        pub fn dlclose(handle: *mut c_void) -> c_int;
-    }
-}
\ No newline at end of file
